<?php

namespace NAPPS\Modules\Smartbanner;

use NAPPS\Contracts\ModuleImplementation;

class SmartbannerModule implements ModuleImplementation {

    private $options;
    public function __construct()
    {
        if ( is_admin() || napps_doing_ajax() ||  napps_doing_cron())
        {
            return;
        }

        add_action( 'template_redirect', array( $this, 'check_show_smartbanner' ) );

    }

    /**
     * Check if customer is not on payment page before adding actions to show smartbanner
     *
     * @return void
     */
    public function check_show_smartbanner() {

        $this->options = get_option('nappsSmartAppBanner');

        $showSmartbanner = true;
        if (isset($_COOKIE['napps_mobile']) && $_COOKIE['napps_mobile'] == 'true') {
            $showSmartbanner = false;
        }
        
        if($showSmartbanner) {
            
            //Hooks
            add_action('wp_head', array($this, 'add_meta'));
            add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts'));
            add_action('wp_footer', array($this, 'wp_footer'));
        }

    }

    /**
     * Add smartbanner load component to footer
     *
     * @return void
     */
    public function wp_footer()
    {
        ?>
        <script type="text/javascript">

            document.addEventListener('smartbanner.view', function() { });
            document.addEventListener('smartbanner.exit', function() { });
            document.addEventListener('smartbanner.clickout', function() { });
            let apiHandler = function() { 
                let content = document.querySelector('meta[name="smartbanner:api"]')
                if(!content) {
                    return;
                }

                if(!window.smartbanner) {
                    setTimeout(() => apiHandler(), 500)
                    return
                }
                
                if(window.smartbanner.height <= 0) {
                    window.smartbanner.publish(); 
                }
            };


            window.addEventListener('DOMContentLoaded', () => {
                apiHandler();
            });

            document.onreadystatechange = () => {
                if (document.readyState === 'complete' || document.readyState === 'interactive') {
                    apiHandler();
                }
            };
            
        </script>
        <?php
    }

    /**
     * Enqueue custom script
     *
     * @return void
     */
    public function enqueue_scripts()
    {
        if (!$this->validOptions()) {
            return;
        }
        
        //Styles
        wp_enqueue_style('smartbanner-css', NAPPS_PLUGIN_DIR_URL . 'public/css/smartbanner.min.css', array(), NAPPS_VERSION, 'all');
        wp_enqueue_style('napps-css', NAPPS_PLUGIN_DIR_URL . 'public/css/napps.css', array(), NAPPS_VERSION, 'all');
        
        //Scripts
        wp_enqueue_script('smartbanner-js', NAPPS_PLUGIN_DIR_URL . 'public/js/smartbanner.min.js', array(), NAPPS_VERSION, false);
    }

    /**
     * Adding the meta tag for android and ios app
     *
     * @return void
     */
    public function add_meta()
    {
        // Check if we have valid options (Title, desc, etc) before adding meta tags to the dom
        if ($this->validOptions()) {

            $current_option = $this->options;

            ?>
                <!-- Start Napps Smart App Banner configuration -->
                <meta name="smartbanner:api" content="true">
                <meta name="smartbanner:title" content="<?php esc_attr_e($current_option['appTitle']) ?>">
                <meta name="smartbanner:author" content="<?php esc_attr_e($current_option['appDesc']) ?>">
                <meta name="smartbanner:icon-apple" content="<?php echo esc_url($current_option['appIcon']) ?>">
                <meta name="smartbanner:icon-google" content="<?php echo esc_url($current_option['appIcon']) ?>">
                <meta name="smartbanner:price" content="<?php _e('FREE', 'napps') ?>">
                <meta name="smartbanner:price-suffix-apple" content="<?php _e(' - On the App Store', 'napps') ?>">
                <meta name="smartbanner:price-suffix-google" content="<?php _e(' - In Google Play', 'napps') ?>">
                <meta name="smartbanner:button" content="<?php _e('VIEW', 'napps') ?>"> 
                <meta name="smartbanner:close-label" content="<?php _e('Close', 'napps') ?>">
            <?php

            $platforms = [];
            // If we have a apple app url
            if (!empty($current_option['appleAppUrl'])) {
                
                array_push($platforms, 'ios');
                $apple_url = esc_url($current_option['appleAppUrl']);
                if(preg_match("/id(\d+)/", $apple_url, $match)) {
                    $apple_ID = $match[1];
                }

                if (isset($apple_ID)) {
                    ?>
                    <!-- Start Smart banner app for Safari on iOS configuration -->
                    <meta name="apple-itunes-app" content="app-id='<?php esc_attr_e($apple_ID) ?>">
                    <!-- End Smart banner app for Safari on iOS configuration -->
                    <?php
                }

                ?> <meta name="smartbanner:button-url-apple" content="<?php echo esc_url($current_option['appleAppUrl']) ?>"> <?php
            }

            // If we have a android app url
            if (!empty($current_option['androidAppUrl'])) {

                array_push($platforms, 'android');
                ?> <meta name="smartbanner:button-url-google" content="<?php echo esc_url($current_option['androidAppUrl']) ?>"> <?php

            }

            if(count($platforms) > 0) {
                ?> <meta name="smartbanner:enabled-platforms" content="<?php echo implode(',', $platforms) ?>"> <?php
            }
            
            ?><!-- End Napps Smart App Banner configuration --><?php
        } 
    }
    
    
    
    /**
     * Check if options has necessary data to show smartbanner
     *
     * @return boolean
     */
    private function validOptions() {
        return !empty($this->options) && 
            !empty($this->options['appTitle']) && 
            !empty($this->options['appDesc']) && 
            !empty($this->options['appIcon']) &&
            (!empty($this->options['appleAppUrl']) || !empty($this->options['androidAppUrl']));
    }

}